"""
OmniHost - High-Performance SSH Management CLI for DevOps Engineers

A feature-rich Python CLI tool for managing SSH connections and executing
commands on remote servers with lightning-fast parallel execution.

Version: 1.0.1
"""

__version__ = "1.0.1"
__author__ = "Sagar Memane <sagarmemane135@github.com>"
__description__ = "High-Performance SSH Management CLI for DevOps Engineers"
__url__ = "https://github.com/sagarmemane135/omnihost"
__license__ = "MIT"

# Public API
__all__ = [
    "__version__",
    "__author__",
    "__description__",
    "__url__",
    "__license__",
]
